#!/bin/bash
#set -x

#this script will make a disk image with the tools needed to update your Recovery HD with the newest OS X Lion release
# Author: Joel Bruner (aka brunerd)

#Changelog
# 081013 Updated by Daniel Shane to make it Mavericks compatible
# ver 12.25.7 - Allows ~ paths to be typed in now, OS version appears in dmg name now, quieter, takes ReocoveryHDUpdate.dmg or pkg, changed verbage to accomodate Lion and Mountain Lion
# ver 1.0 - Initial Release

#Required components:
#Lion Recovery Update v1.0 - http://support.apple.com/downloads/DL1464/en_US/RecoveryHDUpdate.dmg
#"Install Mac OS X Lion.app" - App Store

###########################
# RecoveryHD Updater.command
###########################

#this script that will be saved to the disk image that is created
RecoveryHDUpdaterScript='#!/bin/bash
#brunerd.com 2012

clear

MYPATH="$(dirname "$0")"

#get destination drive
if [ "$1" == "" ]; then
	echo -n "Please drag in DESTINATION disk for Recovery HD partition and press Enter: "
	while [ -z "$DEST" ]; do
	read DEST
	done
else
	DEST="$1"
fi

if [ ! -d "$DEST" ]; then 
	echo "$DEST not found"
	exit
else
	#create Recovery partition
	sudo "$MYPATH"/bin/dmtest ensureRecoveryPartition "$DEST" "$MYPATH"/etc/BaseSystem.dmg 0 0 "$MYPATH"/etc/BaseSystem.chunklist
fi
'
######
# END
######

###########################
# VARIABLES
###########################

#IMAGETEMP is the pathname for the disk image being built, will the OS and Build appended to the name later
IMAGETEMP="/tmp/Recovery HD Updater"

#temp folder for package expansion
RECOVERY_EXPANDED="/tmp/RecoveryHDUpdate"

#Mounted disk image paths 
ESDPATH="/Volumes/Mac OS X Install ESD"
RECOVERYPATH="/Volumes/Mac OS X Lion Recovery HD Update"

#############
# MAIN SCRIPT
#############
#clear;

if [ "$1" == "-h" ]; then
echo "Script function:
This script will create DMG that is able to create/update a recovery partition for any given disk

Usage:
Run with no arguments to be prompted for files and paths

Or run with arguments for automatic disk image creation:
$(basename $0) RecoveryUpdatePath InstallOSXApp DMGDestinationPath
"
exit 0
fi

#get Recovery Update dmg path
if [ "$1" == "" ]; then
	echo -n "Please drag in RecoveryHDUpdate.dmg (or .pkg) and press enter: "
	while [ -z "$RECOVERYHDUPDATE" ]; do
	read RECOVERYHDUPDATE
	done
else
RECOVERYHDUPDATE="$1"
fi

if [ ! -f "$RECOVERYHDUPDATE" ]; then 
echo "$RECOVERYHDUPDATE not found"; exit
fi

#get Lion Installer path
if [ "$2" == "" ]; then
echo -n "Please drag in \"Install (OS X Mavericks)(Mountain Lion)(Lion).app\" and press enter: "
while [ -z "$LION" ]; do
read LION
done
if [ ! -d "$LION" ]; then echo "$LION not found"; exit; fi 
else
LION="$2"
fi


#get destination path for disk image to be created at
if [ "$3" == "" ]; then
echo -n "Please drag in Destination folder for disk image and press enter: "
while [ -z "$DEST" ]; do
read DEST
done

#eval to expand ~ path if typed in
eval DEST=$DEST

if [ ! -d "$DEST" ]; then echo "$DEST is not a valid path"; exit; fi 
else
#eval to expand ~ path if typed in
eval DEST="$3"
fi

if [ "$(basename "$RECOVERYHDUPDATE" | tail -c 4)" == dmg ]; then
	#mount Recovery Update image
	echo "Mounting Recovery HD Update disk image"
	hdiutil attach "$RECOVERYHDUPDATE" -noverify -quiet

	#set path to that of mounted DMG	
	RECOVERYHDUPDATEPATH="$RECOVERYPATH"/RecoveryHDUpdate.pkg
	
elif [ "$(basename "$RECOVERYHDUPDATE" | tail -c 4)" == pkg ]; then
	#set path to that of the pkg
	RECOVERYHDUPDATEPATH="$RECOVERYHDUPDATE"
fi

#expand packge inside to temp folder (contains dmtest)
echo "Expanding RecoveryHDUpdate.pkg"
pkgutil --expand "$RECOVERYHDUPDATEPATH" "$RECOVERY_EXPANDED"

#if we were using what's inside the recovery update we would mount this
#RecoveryHDMeta="/Volumes/Recovery HD Update"
#hdiutil attach "$RECOVERY_EXPANDED"/RecoveryHDUpdate.pkg/RecoveryHDMeta.dmg

#open Lion install ESD image for the newest files
echo "Mounting InstallESD.dmg"
hdiutil attach "$LION"/Contents/SharedSupport/InstallESD.dmg -noverify -quiet


#Check to see if the app is Mavericks and set ESDPATH accordingly
echo "$LION" | grep -i mavericks
if [ $? -eq 0 ]; then
	echo "Mavericks Detected..."
	#get OS version from Current Booted OS
	OSVER=$(defaults read /System/Library/CoreServices/SystemVersion ProductUserVisibleVersion)
	OSBUILD=$(defaults read /System/Library/CoreServices/SystemVersion ProductBuildVersion)
	ESDPATH="/Volumes/OS X Install ESD"
else
	#get OS version from Lion Installer ESD
	OSVER=$(defaults read /System/Library/CoreServices/SystemVersion ProductUserVisibleVersion)
	OSBUILD=$(defaults read /System/Library/CoreServices/SystemVersion ProductBuildVersion)
fi

#append info to path so disk image volume name is informative and useful
IMAGETEMP="$IMAGETEMP $OSVER $OSBUILD"

#make work the folder
mkdir -p "$IMAGETEMP"/etc/
mkdir "$IMAGETEMP"/bin/

#copy dmtest to IMAGETEMP
if [ -f "$RECOVERY_EXPANDED"/RecoveryHDUpdate.pkg/Scripts/Tools/dmtest ]; then
cp "$RECOVERY_EXPANDED"/RecoveryHDUpdate.pkg/Scripts/Tools/dmtest "$IMAGETEMP"/bin/
else
echo "$RECOVERY_EXPANDED/RecoveryHDUpdate.pkg/Scripts/Tools/dmtest not found, exiting"
exit 1
fi

#copy the BaseSystem dmg and chunklist to destination/etc
if [ -f "$ESDPATH"/BaseSystem.chunklist -o -f "$ESDPATH"/BaseSystem.dmg  ]; then
cp "$ESDPATH"/BaseSystem.chunklist "$ESDPATH"/BaseSystem.dmg "$IMAGETEMP"/etc/
#unhide BaseSystem
chflags -R nohidden "$IMAGETEMP"
else
echo "$ESDPATH/BaseSystem* not found, exiting"
fi

#put script in folder root
echo "$RecoveryHDUpdaterScript" > "$IMAGETEMP"/"RecoveryHD Updater.command"
#set permissions
chmod ugo+x "$IMAGETEMP"/"RecoveryHD Updater.command"

#create disk image from folder
echo "Creating RecoveryHDUpdater disk image, please wait"
hdiutil create -srcfolder "$IMAGETEMP" "$DEST"/RecoveryHDUpdater_"$OSVER"_"$OSBUILD".dmg
if [ $? -eq 0 ]; then
echo "Success! Created: $DEST/RecoveryHDUpdater_"$OSVER"_"$OSBUILD".dmg"
echo "Now opening "$DEST"/RecoveryHDUpdater_"$OSVER"_"$OSBUILD".dmg"
hdiutil attach "$DEST"/RecoveryHDUpdater_"$OSVER"_"$OSBUILD".dmg
else
echo "Disk Image failed"
fi

echo "Cleaning Up"
#delete temp folders
rm -rf "$IMAGETEMP" "$RECOVERY_EXPANDED"

#eject the volumes
[ -d "$RECOVERYPATH" ] && hdiutil eject "$RECOVERYPATH"
hdiutil eject "$ESDPATH"

echo "Done."

exit